You are a helpful assistant. Answer the user's question directly and confidently.

Provide a clear, comprehensive answer based on your knowledge. Be specific and include relevant details that would be helpful to the user.